package com.bcxin.backend.core.utils;

import com.bcxin.backend.core.exceptions.SaasBadException;
import com.bcxin.backend.core.exceptions.SaasRetryableException;
import com.github.rholder.retry.Retryer;
import com.github.rholder.retry.RetryerBuilder;
import com.github.rholder.retry.StopStrategies;
import com.github.rholder.retry.WaitStrategies;

import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;

public class RetryUtil {

    public static <T> T  execute(Callable<T> function, int retryCount) {
        try {
            Retryer<T> retryer = RetryerBuilder.<T>newBuilder()
                    .retryIfExceptionOfType(SaasRetryableException.class)
                    .withStopStrategy(StopStrategies.stopAfterAttempt(retryCount))
                    .withWaitStrategy(WaitStrategies.fixedWait(50, TimeUnit.MILLISECONDS))
                    .build();

            return retryer.call(()->{
                try {
                    return function.call();
                }
                catch (Exception e) {
                    if (!ExceptionUtils.getStackMessage(e).contains("504")) {
                        e.printStackTrace();
                        throw new SaasRetryableException(e);
                    } else {
                        e.printStackTrace();
                    }

                    throw e;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();

            throw new SaasBadException(e);
        }
    }

    public static <T> T execute(Callable<T> function) {
        return RetryUtil.execute(function, 5);
    }
}
